#include "traincontrol.h"
#include <zeitgeist/logserver/logserver.h>
#include <zeitgeist/scriptserver/scriptserver.h>
#include <oxygen/agentaspect/agentaspect.h>
#include <oxygen/gamecontrolserver/baseparser.h>
#include <oxygen/gamecontrolserver/predicate.h>

using namespace oxygen;
using namespace zeitgeist;
using namespace std;
using namespace boost;
using namespace salt;

int TrainControl::mIdCounter = 0;

TrainControl::TrainControl()
{
    ++mIdCounter;
    mId = mIdCounter;
    mAgentThinkingTime = 0.0f;
}

TrainControl::~TrainControl()
{
}

void TrainControl::OnLink()
{
    SimControlNode::OnLink();

    mSimulationServer = GetSimulationServer();
    if (mSimulationServer.get() == 0)
    {
        GetLog()->Error()
            << "(TrainControl) ERROR: SimulationServer not found\n";
        return ;
    }

    mGameControlServer = mSimulationServer->GetGameControlServer();
    if (mGameControlServer.get() == 0)
    {
        GetLog()->Error()
            << "(TrainControl) GameControlServer not found." << endl;
    }

    stringstream ss1, ss2;
    ss1 << gServerPath << gTeamName << mId << "/network";
    ss2 << gServerPath << gTeamName << mId << "/behavior";
    mNetwork = shared_dynamic_cast<Network>
        (GetCore()->Get(ss1.str()));
    if (mNetwork.get() == 0)
    {
        GetLog()->Normal()
            << "(TrainControl) Network not found.Entering Debug Mode" << endl;
    }

    mBehavior = shared_dynamic_cast<Behavior>
        (GetCore()->Get(ss2.str()));
    assert(mBehavior);

    mTrainServer = shared_dynamic_cast<TrainServer>
        (GetCore()->Get(gServerPath + "trainserver"));
    assert(mTrainServer);
    
    GetScript()->GetVariable("Agent.ThinkingTime", mAgentThinkingTime);
}

void TrainControl::OnUnlink()
{
    SimControlNode::OnUnlink();

    mSimulationServer.reset();
    mGameControlServer.reset();
    mNetwork.reset();
    mBehavior.reset();
}

void TrainControl::InitSimulation()
{
    if (mNetwork.get() != 0)
    {
        mNetwork->Init();
        mNetwork->PutMessage("(scene rsg/agent/nao/nao.rsg)");
    }
    
    mGameControlServer->AgentConnect(mId);
    shared_ptr<AgentAspect> agentaspect = mGameControlServer->GetAgentAspect(mId);
    shared_ptr<ParameterList> paraList(new ParameterList);
    agentaspect->ImportScene("rsg/agent/nao/nao.rsg", paraList);
}

void TrainControl::DoneSimulation()
{
    if (mNetwork.get() != 0)
    {
        mNetwork->Done();
    }
}

void TrainControl::StartCycle()
{
    if (
         (mSimulationServer.get() == 0) ||
         (mGameControlServer.get() == 0) ||
         //(! mSimulationServer->StepEnabled())
         (mNetwork.get() != 0)
       )
    {
        return ;
    }
    
    shared_ptr<AgentAspect> agent =
        mGameControlServer->GetAgentAspect(mId);
    if (agent.get() == 0)
    {
        GetLog()->Error()
            << "(TrainControl) ERROR: agent not found" << endl;
    }
    else
    {
        agent->RealizeActions
            (mGameControlServer->Parse(mId, mMsgReceive));
    }
}

void TrainControl::EndCycle()
{
    if (
         (mSimulationServer.get() == 0) ||
         (mGameControlServer.get() == 0) ||
         // (! mSimulationServer->StepEnabled())
         (mNetwork.get() != 0)
       )
    {
        return ;
    }
    
    shared_ptr<BaseParser> parser = mGameControlServer->GetParser();
    if (parser.get() == 0)
    {
        GetLog()->Error()
            << "(TrainControl) ERROR: got no parser from "
            << " the GameControlServer" << endl;
        return ;
    }
    
    shared_ptr<AgentAspect> agent =
        mGameControlServer->GetAgentAspect(mId);
    if (agent.get() == 0)
    {
        GetLog()->Error()
            << "(TrainControl) ERROR: agent not found"<< mId << endl;
    }
    else
    {
        shared_ptr<PredicateList> senseList = agent->QueryPerceptors();
        string senses = parser->Generate(senseList);
        if (senses.empty())
        {
        }
        else
        {
            mMsgSend = senses;
        }
    }
}

void TrainControl::SenseAgent()
{
    if (mNetwork.get() != 0)
    {
        if (! mNetwork->GetMessage(mMsg))
        {
            if (mSimulationServer.get() == 0)
            {
                GetLog()->Error()
                    << "(TrainControl) ERROR: SimulationServer not found\n";
                return ;
            }
            else
            {
                mSimulationServer->Quit();
            }
        }
    }
    else
    {
        mMsg = mMsgSend;
    }
}

void TrainControl::ActAgent()
{
    /** default command is null */
    string cmd("");
    if (mBehavior.get() != 0)
    {
        cmd = mBehavior->Think(mMsg);
    }

    mMsgReceive = cmd;

    if (mNetwork.get() != 0)
    {
        while (mTimer.elapsed() * 1000 < mAgentThinkingTime);
        // usleep(10000);

        if (! mNetwork->PutMessage(mMsgReceive))
        {
            if (mSimulationServer.get() == 0)
            {
                GetLog()->Error()
                    << "(TrainControl) ERROR: SimulationServer not found\n";
                return ;
            }
            else
            {
                mSimulationServer->Quit();
            }
        }

        mTimer.restart();
    }
    else
    {
        mTrainServer->Update();
    }
}

