#ifndef TRAINCONTROL_H
#define TRAINCONTROL_H

#include <oxygen/simulationserver/simulationserver.h>
#include <oxygen/simulationserver/simcontrolnode.h>
#include <oxygen/gamecontrolserver/gamecontrolserver.h>
#include <boost/timer.hpp>
#include "utils/network.h"
#include "utils/define.h"
#include "train/trainserver.h"
#include "agent/behavior.h"

class TrainControl : public oxygen::SimControlNode
{
public:
    TrainControl();

    ~TrainControl();

    /** called once when the simulation is started, it notifies
        GameControlServer that a new agent has connected, then
        import the physical model of robot and an AgentAspect
        is givened. At last, ScriptServer does some initial work.
     */
    void InitSimulation();

    /** called once before the simulation is shut down */
    void DoneSimulation();

    /** called when a new simulation cycle starts, before the
        physicial simulation is stepped 
     */
    void StartCycle();
    
    /** called when the current simulation cycle ends */
    void EndCycle();

    /** called each simulation cycle to send sensors'
        information to the agent
     */
    void SenseAgent();

    /** called each simulation cycle to carry out agent actions */
    void ActAgent();

    int GetPlayerId() const { return mId; }

protected:
    void OnLink();

    void OnUnlink();

protected:
    /** cached reference to the Network */
    boost::shared_ptr<Network> mNetwork;
    
    /** cached reference to the Behavior */
    boost::shared_ptr<Behavior> mBehavior;

    /** cached reference to the SimulationServer */
    boost::shared_ptr<oxygen::SimulationServer> mSimulationServer;

    /** cached reference to the GameControlServer */
    boost::shared_ptr<oxygen::GameControlServer> mGameControlServer;

    /** cached reference to the TrainServer */
    boost::shared_ptr<TrainServer> mTrainServer;

    /** timer */
    boost::timer mTimer;

private:
    /** message received from the agent trained */
    std::string mMsgReceive;

    /** message to be sent to the agent trained */
    std::string mMsgSend;

    std::string mMsg;

    /** in debug mode, to mark each agent */
    int mId;

    /** in debug mode, a static integer is needed to count
        the total agents on the field
     */
    static int mIdCounter;

    float mAgentThinkingTime;
};

DECLARE_CLASS(TrainControl);

#endif
