#ifndef TRAINSERVER_H
#define TRAINSERVER_H

#include <zeitgeist/node.h>
#include <oxygen/simulationserver/simulationserver.h>
#include <oxygen/physicsserver/body.h>
#include <oxygen/physicsserver/joint.h>
#include <oxygen/sceneserver/transform.h>
#include "utils/define.h"

class TrainSystem;

/** \class TrainServer provides the interface to access the information
    directly from the internal server, and it holds a trainsystem to
    make certain scene training. 
 */
class TrainServer : public zeitgeist::Node
{
public:
    typedef std::map<RobotPart, std::string> TRobotPartMap;

    typedef std::map<boost::shared_ptr<oxygen::Body>, salt::Matrix> TRobotSceneMap;

    TrainServer();

    ~TrainServer();
    
    bool Init(const std::string& trainSysName);
    
    void Update();

    void ResetTime();

    void Quit();
    
    void SetActiveId(const std::string& id);
    
    std::string GetActiveId() const { return mActiveId; }

    boost::shared_ptr<oxygen::Transform> GetRobotPart(RobotPart part) const;

    boost::shared_ptr<oxygen::Body> GetRobotPartBody(RobotPart part) const;

    boost::shared_ptr<oxygen::Joint> GetRobotPartJoint(RobotPart part) const;

    salt::Vector3f GetRobotPartPos(RobotPart part) const;

    salt::Matrix GetRobotPartMatrix(RobotPart part) const;

    boost::shared_ptr<oxygen::Transform> GetBall() const;

    boost::shared_ptr<oxygen::Body> GetBallBody() const;

    salt::Vector3f GetBallPos() const;

    bool MoveAndRotateAgent(const salt::Vector3f& pos, float angle);

    void DropBall(const salt::Vector3f& pos);
        
    void SetBallVel(const salt::Vector3f& vel);

    bool SaveRobotScene();

    bool RestoreRobotScene();
    
protected:
    void OnLink();

    void OnUnlink();

    void SetupRobotPartMap();

protected:
    std::string mActiveId;

    TRobotPartMap mRobotPartMap;

    /** used to make a scene training */
    TRobotSceneMap mRobotSceneMap;

    /** cached reference to the SimulationServer */
    boost::shared_ptr<oxygen::SimulationServer> mSimulationServer;

    /** cached reference to the TrainSystem */
    boost::shared_ptr<TrainSystem> mTrainSystem;

    /** when a scenario is setted, agents' behavior
        should be delayed to make sure it's static
     */
    int mWaitCycle;
};

DECLARE_CLASS(TrainServer);

#endif

