#ifndef GEOMETRY_H
#define GEOMETRY_H

#include <salt/tvector.h>
#include <salt/matrix.h>
#include <salt/gmath.h>

inline salt::Vector2f MakeVector2f(const salt::Vector3f & v)
{
    return salt::Vector2f(v.x(), v.y());
}

inline float DriectionSame(salt::Vector2f v1,salt::Vector2f v2)
{
	if(v1.x()*v2.x()<0||v1.y()*v2.y()<0)
		return -1.0f;
	return 1.0f;
}
inline salt::Vector2f MakeVector2f(float angleDeg)
{
    return salt::Vector2f(
                          salt::gCos(salt::gDegToRad(angleDeg)),
                          salt::gSin(salt::gDegToRad(angleDeg))
                         );
}

inline float Precision(float value, float remain = 3.0f)
{
    float tmp = salt::gPow(10.0f, remain);
    return int(value * tmp) / tmp;
}

inline float Constrain(float _value, float _min, float _max)
{
    return std::min(_max,std::max(_min,_value));
}

bool            IsRi(const salt::Matrix& R);

bool            InverseMatrix(salt::Matrix& m);

salt::Vector3f  RotToOmega(const salt::Matrix& R);

salt::Matrix    RotationMatrixFrom2Vec(salt::Vector3f from, salt::Vector3f to);

salt::Matrix    Rodrigues(const salt::Vector3f& axis, float theta);

float   Cross(const salt::Vector2f& O, const salt::Vector2f& end1, const salt::Vector2f& end2);
salt::Vector3f Cross(salt::Vector3f x,salt::Vector3f y);

float   Dot  (const salt::Vector2f& O, const salt::Vector2f& end1, const salt::Vector2f& end2);

float   DistanceToLine(const salt::Vector2f& p, const salt::Vector2f& lineStart,
                       const salt::Vector2f& lineEnd, salt::Vector2f& proj);

float   GetProjectionDistance(const salt::Vector2f& p, const salt::Vector2f& lineStart,
                              const salt::Vector2f& lineEnd);

bool   IsObjectInTheWay(const salt::Vector2f& objectPos, const salt::Vector2f& lineStart,
                         const salt::Vector2f& lineEnd, float radius);

float   GetVector2fAngleDeg(const salt::Vector2f &v);

float   GetMatrixRotateXRad(const salt::Matrix &m);

float   GetMatrixRotateYRad(const salt::Matrix &m);

float   GetMatrixRotateZRad(const salt::Matrix &m);
inline float
SIGN(float a, float b)
{
    if (b > 0) return fabs(a); else return -fabs(a);
}

bool    svd(float* X, const float** A, const float* B, int M, int N);
bool IsVecInBetween(salt::Vector2f to,salt::Vector2f from ,salt::Vector2f unknown);

bool TurnToClockWise(salt::Vector2f &vec1,salt::Vector2f &vec2);


#endif
