#ifndef __SMP_DATASTRUCT_H__
#define __SMP_DATASTRUCT_H__
#include "smp_func.h"

/*	Timed Data	*/
template <class Data> class TimedData{
public:
	Data data;
	int time;

	TimedData(int time = -2){ this->time = time;}
	TimedData(const Data& data, int time){ this->data = data; this->time = time;}

	void Setdata(const Data& data, int time){ this->data = data; this->time = time;}
	void InvalidTime(){time = -1;}

	void operator = (const Data& d){ data = d;}
	TimedData operator + (TimedData d){ return TimedData(data + d, time);}
	TimedData operator - (TimedData d){ return TimedData(data - d, time);}
	TimedData operator / (TimedData d){ return TimedData(data / d, time);}
	TimedData operator * (TimedData d){ return TimedData(data * d, time);}
};

/*************	TimedDataArray	**********************/
/*洢:ʱT, 
: ʱ
*/
template <class T, int size> class TimedDataArray{
private:
	typedef ::TimedData<T> _Tdata;
	_Tdata data[size];
	inline int Index(int idx){return idx % size >=0 ? idx % size : idx % size + size;}
public:
	inline _Tdata& TimedData(int time){return data[Index(time)];}
	inline T& Data(int time){return data[Index(time)].data;}
	inline T ChkData(int time){return IsDataKnown(time) ? data[Index(time)].data : T(0);}
	inline bool IsDataKnown(int time){return data[Index(time)].time == time;}
	inline void Setdata(const T& data, int time){this->data[Index(time)].data = data; this->data[Index(time)].time = time;}
};

/*洢: ʱ, СΪTsize
: ʱ+
*/
template <class T, int Tsize, int size> class Timed2D_Array{
private:
	TimedData<T[Tsize]> data[size];
	inline int Index(int idx){return idx % size >=0 ? idx % size : idx % size + size;}
public:	
	//inline T[]& Data(int time){return data[Index(time)].data;}	
	inline T& Data(int idx, int time){return data[Index(time)].data[idx];}
	inline bool IsDataKnown(int time){return data[Index(time)].time == time;}
	inline void Setdata(const T& data, int idx, int time){this->data[Index(time)].data[idx] = data; this->data[Index(time)].time = time;}
};

/*洢: ʱĶ
: ʱ + (idx1, idx2)
*/
template <class T, int size> class TimedDataTri2DArray{
private:
	TimedData<T> data[size * (size -1) /2 + 1];
	inline int Index(int idx1, int idx2){
		if (!IsValidIdx(idx1, idx2))
			return size * (size -1) /2;//invalid unit

		if (idx1 > idx2)
			Swap(idx1, idx2);
		return (2 * size - 1  - idx1) * idx1 /2 + idx2 - idx1 - 1;
	}
public:
	inline bool IsValidIdx(int idx1, int idx2){
		if (idx1 < 0 || idx1 >= size){
			//wrong idx-1
			return false;
		}

		if (idx2 < 0 || idx2 >= size){
			//wrong idx-2
			return false;
		}

		if (idx1 == idx2){
			//wrong
			return false;
		}
		return true;
	}

	bool IsDataKnown(int idx1, int idx2, int time){
		return data[Index(idx1, idx2)].time == time;
	}

	inline void SetData(const T& Data, int idx1, int idx2, int time){
		data[Index(idx1, idx2)].data = Data;
		data[Index(idx1, idx2)].time = time;
	}

	inline void SetTime(int idx1, int idx2, int time){
		data[Index(idx1, idx2)].time = time;
	}

	inline TimedData<T>& Element(int idx1, int idx2){ return data[Index(idx1, idx2)];}

	inline T& Data(int idx1, int idx2){return data[Index(idx1, idx2)].data;}
};

#endif //__SMP_DATASTRUCT_H__
